package ru.amse.baltijsky.javascheme.nodeshape;

import ru.amse.baltijsky.javascheme.util.Dir;
import ru.amse.baltijsky.javascheme.util.VisConst;

import java.awt.*;

/**
 * A shape that displays a branch between the parent and the child, continuing the branch to the right.
 * It does not correspond to any schema node and is created only to achieve layouting results for
 * writing conditions over the edges from branching schema tree nodes to their children. Continued means that
 * another child of the same parent exists that goes to the right of this child, so the line will continue to it seamlessly.
 */
public class ContinuedBranchShape extends BranchShape {
    /**
     * Creates a new shape that contains the specified code.
     *
     * @param code the code
     */
    public ContinuedBranchShape(String code) {
        super(code);
    }

    public void draw(int x, int y, Graphics g) {
        int[] dims = getShapeDims(g);
        Color oldColor = g.getColor();

        // draw the branch
        g.setColor(VisConst.SCHEME_COL);
        g.drawLine(x - dims[Dir.l()], y, x + dims[Dir.r()], y);
        g.drawLine(x, y, x, y + dims[Dir.d()]);
        // draw the label
        g.setColor(VisConst.TEXT_COL);
        g.drawString(getCode(), x - dims[Dir.l()] + getOffset().x, y - getOffset().y);

        g.setColor(oldColor);
    }
}