package ru.amse.baltijsky.javascheme.nodeshape;

import ru.amse.baltijsky.javascheme.util.Dir;
import ru.amse.baltijsky.javascheme.util.VisConst;

import java.awt.*;

/**
 * A shape that displays a branch between the parent and the child, without any branches to the right.
 * It does not correspond to any schema node and is created only to achieve layouting results for
 * writing conditions over the edges from branching schema tree nodes to their children. Final means that
 * the child for which this shape is created is the last child of its parent.
 */
public class LastBranchShape extends BranchShape {

    /**
     * Internal constructor, used to encapsulate the work with the code string and calculate node dimensions.
     *
     * @param code the code of the node
     */
    public LastBranchShape(String code) {
        super(code);
    }

    public void draw(int x, int y, Graphics g) {
        int[] dims = getShapeDims(g);
        Color oldColor = g.getColor();

        // draw the branch
        g.setColor(VisConst.SCHEME_COL);
        g.drawLine(x - dims[Dir.l()], y, x, y);
        g.drawLine(x, y, x, y + dims[Dir.d()]);
        // draw the label
        g.setColor(VisConst.TEXT_COL);
        g.drawString(getCode(), x - dims[Dir.l()] + getOffset().x, y - getOffset().y);

        g.setColor(oldColor);
    }
}
