package ru.amse.baltijsky.javascheme.nodeshape;

import ru.amse.baltijsky.javascheme.util.Dir;
import static ru.amse.baltijsky.javascheme.util.Dir.*;
import ru.amse.baltijsky.javascheme.util.Size;
import ru.amse.baltijsky.javascheme.util.VisConst;

import java.awt.*;

/**
 * A switch node shape  a rectangle with an isosceles triangle (ear) at the bottom.
 */
public class SwitchNodeShape extends AbstractNodeShape {
    /**
     * One of the two equal ear angles.
     */
    private static final double ALPHA = Math.PI / 6;
    /**
     * Code offset inside the code frame.
     */
    private static final Size OFFSET = new Size(5, 5);

    /**
     * Internal constructor, used to encapsulate the work with the code string and calculate node dimensions.
     *
     * @param code the code of the node
     */
    public SwitchNodeShape(String code) {
        super(code);
    }

    public void draw(int x, int y, Graphics g) {
        int[] dims = getShapeDims(g);
        int ear = (int) ((dims[l()] + dims[r()]) / 2 * Math.tan(ALPHA));

        Color oldColor = g.getColor();
        g.setColor(VisConst.SCHEME_COL);
        g.drawLine(x - dims[l()], y - dims[u()], x + dims[r()], y - dims[u()]);
        g.drawLine(x + dims[r()], y - dims[u()], x + dims[r()], y + dims[d()] - ear);
        g.drawLine(x + dims[r()], y + dims[d()] - ear, x, y + dims[d()]);
        g.drawLine(x, y + dims[d()], x - dims[l()], y + dims[d()] - ear);
        g.drawLine(x - dims[l()], y + dims[d()] - ear, x - dims[l()], y - dims[u()]);

        g.setColor(VisConst.TEXT_COL);
        g.drawString(getCode(), x - dims[l()] + OFFSET.x, y + OFFSET.y);

        g.setColor(oldColor);
    }

    public Size getBoundRectSize(Graphics g) {
        int[] dims = getShapeDims(g);
        return new Size(dims[l()] + dims[r()], dims[u()] + dims[d()]);
    }

    @Override
    public int[] getShapeDims(Graphics g) {
        Size codeSize = getCodeSize(g);
        int basicSizeX = OFFSET.x + codeSize.x + OFFSET.x;
        int basicSizeY = OFFSET.y + codeSize.y + OFFSET.y;
        int[] ret = new int[Dir.length()];
        ret[l()] = basicSizeX / 2;
        ret[r()] = basicSizeX - ret[l()];
        ret[u()] = basicSizeY / 2;
        ret[d()] = basicSizeY + (int) (basicSizeX / 2 * Math.tan(ALPHA)) - ret[u()];
        return ret;
    }
}