package ru.amse.baltijsky.javascheme.tree;

import ru.amse.baltijsky.javascheme.nodeshape.DoWhileClosingNodeShape;
import ru.amse.baltijsky.javascheme.nodeshape.DoWhileNodeShape;
import ru.amse.baltijsky.javascheme.nodeshape.INodeShape;

/**
 * Represents a postconditional do-while loop.
 * May have several previous nodes (join) and one or zero next nodes (zero iff it is the last node in current nesting scope).
 */
public class DoWhileNode extends NestingNode implements ILoopNode {
    /**
     * Default constructor. Creates a node without previous or parent (nesting) node.
     */
    public DoWhileNode() {
        super();
    }

    public DoWhileNode(SchemaNode parentNode, SchemaNode prevNode) {
        super(parentNode, prevNode);
    }

    /**
     * Constructs a new node without previous or nesting node and with information about the corresponding code.
     *
     * @param s code associated with the node
     */
    public DoWhileNode(String[] s) {
        super(s);
    }

    /**
     * Constructs a new node with full information about the nesting and previous nodes and the corresponding code.
     *
     * @param parentNode link to the parent (nesting) node
     * @param prevNode   link to the node that is previous in the sense of program flow
     * @param s          code associated with the node
     */
    public DoWhileNode(SchemaNode parentNode, SchemaNode prevNode, String[] s) {
        super(parentNode, prevNode, s);
    }

    public INodeShape getShape() {
        return new DoWhileNodeShape("do");
    }

    @Override
    public INodeShape getClosingShape() {
        return new DoWhileClosingNodeShape(getCodeAsString());
    }

    public NodeType getNodeType() {
        return NodeType.DO_WHILE;
    }
}
